package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import org.apache.commons.lang3.Validate;

public class VDmnDecisionImpl extends AbstractVDmnElement implements VDmnDecision {

  private VDmnDefinition dmnDefinition;
  private VDmnDecisionTable dmnDecisionTable;

  private VDmnDecisionImpl() {}

  @Override
  public VDmnDefinition getDmnDefinition() {
    return dmnDefinition;
  }

  @Override
  public VDmnDecisionTable getDmnDecisionTable() {
    return dmnDecisionTable;
  }

  public static Builder getBuilder() {
    return new VDmnDecisionImpl().new Builder();
  }

  public class Builder extends DmnElementBuilder<VDmnDecisionImpl> {

    public Builder withDmnDefinition(VDmnDefinition dmnDefinition) {
      value.dmnDefinition = dmnDefinition;
      return this;
    }

    public Builder withDecisionId(DecisionId decisionId) {
      value.id = decisionId;
      return this;
    }

    public Builder withDmnDecisionTable(VDmnDecisionTable dmnDecisionTable) {
      value.dmnDecisionTable = dmnDecisionTable;
      return this;
    }

    public Builder withDmnDecisionTableFromBuilder(
        VDmnDecisionTableImpl.Builder dmnDecisionTableBuilder) {
      withDmnDecisionTable(dmnDecisionTableBuilder.getUnbuildValue());
      dmnDecisionTableBuilder.withDmnDecision(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.dmnDefinition);
      Validate.notNull(value.id);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
