package de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils;

import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.apache.commons.lang3.Validate;

public class VDmnFunctions {

  public static List<VDmnColumn> getColumnsByTypeRef(
      VDmnDefinition dmnDefinition, VTypeRef vTypeRef) {
    Validate.notNull(dmnDefinition);
    Validate.notNull(vTypeRef);
    List<VDmnColumn> columns = new ArrayList<>();
    columns.addAll(
        dmnDefinition.getDmnDecisions().stream()
            .flatMap(
                v ->
                    v.getDmnDecisionTable().getInputColumns().stream()
                        .filter(c -> c.getTypeRef() == vTypeRef))
            .collect(Collectors.toList()));
    columns.addAll(
        dmnDefinition.getDmnDecisions().stream()
            .flatMap(
                v ->
                    v.getDmnDecisionTable().getOutputColumns().stream()
                        .filter(c -> c.getTypeRef() == vTypeRef))
            .collect(Collectors.toList()));
    return columns;
  }

  public static List<VDmnOutputColumn> getOutputColumns(VDmnDefinition dmnDefinition) {
    return Validate.notNull(dmnDefinition).getDmnDecisions().stream()
        .flatMap(v -> v.getDmnDecisionTable().getOutputColumns().stream())
        .collect(Collectors.toList());
  }

  public static List<VDmnValue> getColumnValuesInRules(VDmnColumn column, List<VDmnRule> rules) {
    return getColumnValuesInRuleIds(
        column, rules.stream().map(VDmnRule::getRuleId).collect(Collectors.toList()));
  }

  public static List<VDmnValue> getColumnValuesInRuleIds(VDmnColumn column, List<RuleId> ruleIds) {
    return column.getValues().stream()
        .filter(v -> ruleIds.contains(v.getRuleId()))
        .collect(Collectors.toList());
  }

  public static List<VDmnInputValue> getColumnValuesInRules(
      VDmnInputColumn column, List<VDmnRule> rules) {
    return getColumnValuesInRuleIds(
        column, rules.stream().map(VDmnRule::getRuleId).collect(Collectors.toList()));
  }

  public static List<VDmnInputValue> getColumnValuesInRuleIds(
      VDmnInputColumn column, List<RuleId> ruleIds) {
    return column.getInputValues().stream()
        .filter(v -> ruleIds.contains(v.getRuleId()))
        .collect(Collectors.toList());
  }

  public static boolean differentConclusions(List<VDmnRule> rules) {
    if (rules.size() == 0) {
      return false;
    }
    VDmnRule refRule = rules.get(0);
    for (VDmnRule rule : rules) {
      if (differentConclusions(refRule, rule)) {
        return true;
      }
    }
    return false;
  }

  public static boolean differentConclusions(VDmnRule oneRule, VDmnRule otherRule) {
    if (oneRule.getDmnOutputValues().size() != otherRule.getDmnOutputValues().size()) {
      return true;
    }
    for (int i = 0; i < oneRule.getDmnOutputValues().size(); i++) {
      if (oneRule
          .getDmnOutputValues()
          .get(i)
          .getText()
          .equals(otherRule.getDmnOutputValues().get(0).getText())) {
        return true;
      }
    }
    return false;
  }

  public static List<List<VDmnRule>> getRuleClustersWithIdenticalOutput(
      VDmnDecisionTable dmnDecisionTable) {
    List<List<VDmnRule>> clusters = new ArrayList<>();
    List<VDmnRule> remainingRules = new ArrayList<>(dmnDecisionTable.getRules());
    for (int i = 0; i < remainingRules.size(); i++) {
      List<VDmnRule> newCluster = new ArrayList<>();
      if (remainingRules.get(i) != null) {
        newCluster.add(remainingRules.get(i));
        remainingRules.set(i, null);
        for (int u = i + 1; u < remainingRules.size(); u++) {
          if (remainingRules.get(u) != null
              && !differentConclusions(newCluster.get(0), remainingRules.get(u))) {
            newCluster.add(remainingRules.get(u));
            remainingRules.set(u, null);
          }
        }
        if (newCluster.size() > 1) {
          clusters.add(newCluster);
        }
      }
    }
    return clusters;
  }
}
