package de.unikoblenz.fgbks.core.dmn.verification.result;

import de.unikoblenz.fgbks.base.domain.Message;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerifierResult.Builder;
import org.apache.commons.lang3.Validate;

public class VerificationResultEntryFactory {

  private VerifierResult.Builder verifierResultBuilder;
  private VerificationResultEntry.Builder verificationResultEntryBuilder;

  private VerificationResultEntryFactory(Builder verifierResultBuilder) {
    this.verifierResultBuilder = Validate.notNull(verifierResultBuilder);
    verificationResultEntryBuilder = null;
  }

  public static VerificationResultEntryFactory create(Builder verifierResultBuilder) {
    return new VerificationResultEntryFactory(verifierResultBuilder);
  }

  private VerificationResultEntry.Builder getCurrentOrCreate() {
    if (verificationResultEntryBuilder == null) {
      verificationResultEntryBuilder = VerificationResultEntry.getBuilder();
    }
    return verificationResultEntryBuilder;
  }

  public VerificationResultEntryFactory addElement(
      VerificationResultEntryElement verificationResultEntryElement) {
    getCurrentOrCreate().addVerificationResultEntryElement(verificationResultEntryElement);
    return this;
  }

  public void addToEntry(VerificationClassification verificationClassification, String message) {
    addToEntry(verificationClassification, message, null);
  }

  public void addToEntry(
      VerificationClassification verificationClassification, String message, Object... args) {
    addToEntry(verificationClassification, new Message(String.format(message, args)));
  }

  public void addToEntry(VerificationClassification verificationClassification, Message message) {
    Validate.notNull(
        verificationResultEntryBuilder,
        "First call \"addElement\" in VerificationResultEntryFactory before calling \"addToEntry\"");
    verificationResultEntryBuilder.withClassification(verificationClassification);
    verificationResultEntryBuilder.withMessage(new Message(message));
    verifierResultBuilder.addVerificationResultEntry(verificationResultEntryBuilder.build());
    verificationResultEntryBuilder = null;
  }
}
